@@ -108,6 +108,7 @@ global asset_inventory_limit_hosts
 global asset_inventory_table_name
 global asset_inventory_data_files
 global asset_inventory_temp_shelve_file
+global asset_inventory_present_csv_cell_as_json
 
 # Initialize Some Globals
 
@@ -631,6 +632,7 @@ def get_asset_inventory_user_config():
     global asset_inventory_concurrency_limit
     global asset_inventory_multi_proc_batch_size
     global asset_inventory_limit_hosts
+    global asset_inventory_present_csv_cell_as_json
     global asset_inventory_csv_truncate_cell_limit
     global qetl_user_config_settings_yaml_file
     global qetl_manage_user_selected_datetime
@@ -655,6 +657,11 @@ def get_asset_inventory_user_config():
     asset_inventory_limit_hosts = \
         etld_lib_config_settings_yaml_dict.get('asset_inventory_limit_hosts')
 
+    if 'asset_inventory_present_csv_cell_as_json' in etld_lib_config_settings_yaml_dict:
+        asset_inventory_present_csv_cell_as_json = etld_lib_config_settings_yaml_dict.get('asset_inventory_present_csv_cell_as_json')
+    else:
+        asset_inventory_present_csv_cell_as_json = False
+
     etld_lib_functions.logger.info(f"asset inventory config - {qetl_user_config_settings_yaml_file}")
     etld_lib_functions.logger.info(f"asset_inventory_export_dir yaml - {asset_inventory_export_dir}")
     etld_lib_functions.logger.info(f"asset_inventory_json_dir yaml - {asset_inventory_json_dir}")
