@@ -89,7 +89,7 @@ def flatten_dictionary(data):
 
 def truncate_csv_cell(max_length=32700, csv_cell="", truncated_field_list="", csv_column=""):
     if len(csv_cell) > max_length:
-        truncated_field_list = f"{truncated_field_list}{len(csv_cell):09d}: is Length of {csv_column}, truncated at: {max_length:,}\n"
+        truncated_field_list = f"{truncated_field_list}{csv_column}:{len(csv_cell):,} truncated_at: {max_length:,}\n"
         csv_cell = csv_cell[:max_length]
 
     return csv_cell, truncated_field_list
