@@ -34,6 +34,7 @@ def create_table(table_name, csv_columns, sqlite_file, key):
 
 def bulk_insert_csv_file(table_name, csv_file_name, csv_columns, sqlite_file):
     global count_rows_added_to_table
+    count_rows_added_to_table_display_progress = 0
     csv.field_size_limit(sys.maxsize)
     # etld_lib_functions.logger.info(f"csv.field_size_limit(sys.maxsize): {str(sys.maxsize)}")
     try:
@@ -57,7 +58,12 @@ def bulk_insert_csv_file(table_name, csv_file_name, csv_columns, sqlite_file):
                     cursor_obj.executemany(f"INSERT INTO {table_name} VALUES({values_var})", csv_row_list)
                     conn.commit()
                     csv_row_list = []
-                count_rows_added_to_table = count_rows_added_to_table + 1
+                count_rows_added_to_table += 1
+                count_rows_added_to_table_display_progress += 1
+                if count_rows_added_to_table_display_progress > 9999:
+                    etld_lib_functions.logger.info(f"sqlite rows written: {count_rows_added_to_table}")
+                    count_rows_added_to_table_display_progress = 0
+
             if len(csv_row_list) > 0:
                 cursor_obj.executemany(f"INSERT INTO {table_name} VALUES({values_var})", csv_row_list)
                 conn.commit()
